#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from airflow.models import DAG

from airflow.operators.bash_operator import BashOperator
from airflow.operators.dagrun_operator import TriggerDagRunOperator
from airflow.utils.trigger_rule import TriggerRule

import pendulum
from datetime import timedelta, datetime

# time zone을 한국 시간으로 설정하려면 pendulum 등의 library를 이용한다.
local_tz = pendulum.timezone("Asia/Seoul")


"""
tirgger_dag_run.py는 처음에 BashOperator를 사용하여 현재 시각을 표시하고, 다음으로 trigger의 함수를 정의한 후에 TriggerDagRunOperator에 의해 그 함수를 호출한 다음 tirggered_dag를 실행하는 DAG다.
"""

# DAG 객체로 이용할 공통된 인수를 정의한다.
default_args = {
    'owner': 'Airflow',
    'start_date': datetime(2019, 1, 1, tzinfo=pendulum.timezone('Asia/Seoul')),
    'depends_on_past': True,
    'retries': 1,
    'retry_delay': timedelta(minutes=5),
}

# DAG 객체를 정의한다.
# 여기에서는 정기 실행의 정의（schedule_interval）를 '1회만'으로 하고 있다.
dag = DAG(
    dag_id='tirgger_dag_run',
    default_args=default_args,
    schedule_interval='@once',
)


# 우선은 BashOperator를 사용하여 현재 시각을 표시한다.
operator_1 = BashOperator(
    task_id='operator_1',
    bash_command='echo {}'.format(datetime.now()),
    dag=dag,
)

"""
TriggerDagRunOperator는 지정한 dag_id의 DAG를 실행시키는 Operator다.
파라미터에는 다음의 것이 있다.
・trigger_dag_id：트리거되는 측의 dag_id (문자열형)
・python_callable：context, dag_run_obj라는 ２개의 인수를 가진 함수를 정의하고 그 함수를 지정한다. 반환값은 ‘return dag_run_obj’이라고 한다. dag_run_obj는 run_id와 payload 2가지 속성을 가진다. ‘dag_run_obj.payload = {‘message’: trigger_
dag_id}’는 트리거되는 측에 파라미터를 건넨다.
＊참조원；https://airflow.readthedocs.io/en/1.10.2/code.html#airflow.operators.dagrun_operator.TriggerDagRunOperator
"""

# trigger용 함수를 정의한다.
# 인수로서 context, dag_run_obj를 이용하여 ALL_DONE으로 실행된다.


def trigger(context, dag_run_obj):
    dag_run_obj.payload = {"message": context["params"]["message"]}
    return dag_run_obj


# Trigger_rule을 이용하여 ALL_DONE으로 실행시킨다.
# Trigger_rule은 task끼리의 의존관계에 다양한 룰을 설정할 수 있는 구조다.
# 디폴트에서는 상류가 모두 성공(success)한 경우에 다음 task가 실행된다.
# https://airflow.readthedocs.io/en/1.10.2/concepts.html#trigger-rules
task_tirgger_dag_run = TriggerDagRunOperator(
    task_id="task_tirgger_dag_run",
    trigger_dag_id="tirggered_dag",
    python_callable=trigger,
    provide_context=True,
    params={
        "message": "Hello world !"
    },
    trigger_rule=TriggerRule.ALL_DONE,
    dag=dag,
)

operator_1 >> task_tirgger_dag_run
